@@ -21,15 +21,17 @@ class IBKRCashOp:
 
 
 # -----------------------------------------------------------------------------------------------------------------------
-#TODO Check dividend parsing for "RDS B" stock (see example here https://github.com/KonishchevDmitry/investments/issues/28 )
 class IBKR:
     BondPricipal = 1000
     CancelledFlag = 'Ca'
-    TaxNotePattern = "^(.*) - (..) TAX$"
-    MergerPattern = "^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +MERGED\(\w+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
-    SpinOffPattern = "^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPINOFF +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
-    SymbolChangePattern = "^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +CUSIP\/ISIN CHANGE TO +\((?P<isin_new>\w+)\) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
-    SplitPattern = "^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPLIT +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+    PaymentInLiueOfDividend = 'PAYMENT IN LIEU OF DIVIDEND'
+    TaxFullPattern = r"^(?P<description>.*) - (?P<country>\w\w) TAX$"
+    TaxNotePattern = r"^(?P<symbol>.*\w) ?\((?P<isin>\w+)\)(?P<prefix>( \w*)+) +(?P<amount>\d+\.\d+)?(?P<suffix>.*)$"
+    DividendNotePattern = r"^(?P<symbol>.*\w) ?\((?P<isin>\w+)\)(?P<prefix>( \w*)+) +(?P<amount>\d+\.\d+)?(?P<suffix>.*) \(.*\)$"
+    MergerPattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +MERGED\(\w+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+    SpinOffPattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPINOFF +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+    SymbolChangePattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +CUSIP\/ISIN CHANGE TO +\((?P<isin_new>\w+)\) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+    SplitPattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPLIT +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
 
     AssetType = {
         'CASH': PredefinedAsset.Money,
@@ -52,15 +54,16 @@ class IBKR:
         self._parent = parent
         self._filename = filename
         self.last_selected_account = None
+        self._settled_cash = {}
 
     @staticmethod
-    def flString(data, name, default_value, _caller):
+    def flString(data, name, default_value):
         if name not in data.attrib:
             return default_value
         return data.attrib[name]
 
     @staticmethod
-    def flNumber(data, name, default_value, _caller):
+    def flNumber(data, name, default_value):
         if name not in data.attrib:
             return default_value
         try:
@@ -70,7 +73,7 @@ class IBKR:
         return value
 
     @staticmethod
-    def flTimestamp(data, name, default_value, _caller):
+    def flTimestamp(data, name, default_value):
         if name not in data.attrib:
             return default_value
         time_str = data.attrib[name]
@@ -86,7 +89,7 @@ class IBKR:
             return None
 
     @staticmethod
-    def flAssetType(data, name, default_value, _caller):
+    def flAssetType(data, name, default_value):
         if name not in data.attrib:
             return default_value
         try:
@@ -99,7 +102,7 @@ class IBKR:
                 return None
 
     @staticmethod
-    def flCorpActionType(data, name, default_value, _caller):
+    def flCorpActionType(data, name, default_value):
         if name not in data.attrib:
             return default_value
         try:
@@ -112,7 +115,7 @@ class IBKR:
                 return None
 
     @staticmethod
-    def flCashOpType(data, name, default_value, _caller):
+    def flCashOpType(data, name, default_value):
         operations = {
             'Dividends':                    IBKRCashOp.Dividend,
             'Payment In Lieu Of Dividends': IBKRCashOp.Dividend,
@@ -138,10 +141,10 @@ class IBKR:
                 return None
 
     @staticmethod
-    def flAccount(data, name, default_value, caller):
+    def flAccount(data, name, default_value):
         if name not in data.attrib:
             return default_value
-        if data.tag == 'Trade' and IBKR.flAssetType(data, 'assetCategory', None, None) == PredefinedAsset.Money:
+        if data.tag == 'Trade' and IBKR.flAssetType(data, 'assetCategory', None) == PredefinedAsset.Money:
             if 'symbol' not in data.attrib:
                 logging.error(g_tr('IBKR', "Can't get currencies for accounts: ") + f"{data}")
                 return None
@@ -152,7 +155,7 @@ class IBKR:
             currencies.append(data.attrib['ibCommissionCurrency'])
             accountIds = []
             for currency in currencies:
-                account = caller.findAccountID(data.attrib[name], currency)
+                account = JalDB().get_account_id(data.attrib[name], currency)
                 if account is None:
                     return None
                 accountIds.append(account)
@@ -161,27 +164,30 @@ class IBKR:
             if default_value is None:
                 logging.error(g_tr('IBKR', "Can't get account currency for account: ") + f"{data}")
             return default_value
-        return caller.findAccountID(data.attrib[name], data.attrib['currency'])
+        return JalDB().get_account_id(data.attrib[name], data.attrib['currency'])
 
     @staticmethod
-    def flAsset(data, name, default_value, caller):
+    def flAsset(data, name, default_value):
         if name not in data.attrib:
             return default_value
         if data.attrib[name] == '':
             return default_value
-        if data.tag == 'Trade' and IBKR.flAssetType(data, 'assetCategory', None, None) == PredefinedAsset.Money:
+        if data.tag == 'Trade' and IBKR.flAssetType(data, 'assetCategory', None) == PredefinedAsset.Money:
             currency_asset = default_value
             for currency in data.attrib['symbol'].split('.'):
-                currency_asset = caller.findAssetID(currency)
+                currency_asset = JalDB().get_asset_id(currency)
             return currency_asset
 
         isin = data.attrib['isin'] if 'isin' in data.attrib else ''
         if data.tag == 'CorporateAction' and data.attrib[name].endswith('.OLD'):
-            return caller.findAssetID(data.attrib[name][:-len('.OLD')], isin=isin)
-        return caller.findAssetID(data.attrib[name], isin=isin)
+            return JalDB().get_asset_id(data.attrib[name][:-len('.OLD')], isin=isin)
+        return JalDB().get_asset_id(data.attrib[name], isin=isin)
 
     def load(self):
+        self._settled_cash = {}
+
         section_loaders = {
+            'CashReport': self.loadIBBalances,
             'SecuritiesInfo': self.loadIBSecurities,  # Order of load is important - SecuritiesInfo is first
             'Trades': self.loadIBTrades,
             'OptionEAE': self.loadIBOptions,
@@ -208,13 +214,20 @@ class IBKR:
                                 return False
                             section_loaders[section](section_data)
         except Exception as e:
-            logging.error(g_tr('StatementLoader', "Failed to parse Interactive Brokers flex-report") + f": {e}")
+            logging.error(g_tr('StatementLoader', "Failed to parse Interactive Brokers flex-report") + f": {e}", exc_info = True)
             return False
         logging.info(g_tr('StatementLoader', "IB Flex-statement loaded successfully"))
+        for account in self._settled_cash:
+            logging.info(g_tr('StatementLoader', 'Planned cash: ') + f"{self._settled_cash[account]:.2f} " +
+                              f"{JalDB().get_asset_name(JalDB().get_account_currency(account))}")
         return True
 
     def getIBdata(self, section):
         section_descriptions = {
+            'CashReport': {'tag': 'CashReportCurrency',
+                           'level': 'Currency',
+                           'values': [('accountId', IBKR.flAccount, None),
+                                      ('endingSettledCash', IBKR.flNumber, None)]},
             'SecuritiesInfo': {'tag': 'SecurityInfo',
                                'level': '',
                                'values': [('symbol', IBKR.flString, None),
@@ -291,10 +304,10 @@ class IBKR:
         for sample in section.xpath(tag):
             tag_dictionary = {}
             if section_descriptions[section.tag]['level']:  # Skip extra lines (SUMMARY, etc)
-                if IBKR.flString(sample, 'levelOfDetail', '', self) != section_descriptions[section.tag]['level']:
+                if IBKR.flString(sample, 'levelOfDetail', '') != section_descriptions[section.tag]['level']:
                     continue
             for attr_name, attr_loader, attr_default in section_descriptions[section.tag]['values']:
-                attr_value = attr_loader(sample, attr_name, attr_default, self._parent)
+                attr_value = attr_loader(sample, attr_name, attr_default)
                 if attr_value is None:
                     logging.error(
                         g_tr('StatementLoader', "Failed to load attribute: ") + f"{attr_name} / {sample.attrib}")
@@ -303,12 +316,16 @@ class IBKR:
             data.append(tag_dictionary)
         return data
 
+    def loadIBBalances(self, balances):
+        for balance in balances:
+            self._settled_cash[balance['accountId']] = balance['endingSettledCash']
+
     def loadIBSecurities(self, assets):
         cnt = 0
         for asset in assets:
             # IB may use '.OLD' suffix if asset is being replaced
             symbol = asset['symbol'][:-len('.OLD')] if asset['symbol'].endswith('.OLD') else asset['symbol']
-            asset_id = self._parent.findAssetID(symbol, asset['isin'], dialog_new=False)
+            asset_id = JalDB().get_asset_id(symbol, isin=asset['isin'], dialog_new=False)
             if asset_id is not None:
                 continue
             asset_type = PredefinedAsset.ETF if asset['subCategory'] == "ETF" else asset['assetCategory']
@@ -374,7 +391,7 @@ class IBKR:
                     logging.error(g_tr('StatementLoader', "Merger description miss some data ") + f"'{action}'")
                     continue
                 description_b = action['description'][:parts.span(6)[0]] + merger_a['symbol_old'] + ", "
-                asset_b = self._parent.findAssetID(merger_a['symbol_old'], merger_a['isin_old'])
+                asset_b = JalDB().get_asset_id(merger_a['symbol_old'], isin=merger_a['isin_old'])
 
                 paired_record = list(filter(
                     lambda pair: pair['symbol'] == asset_b
@@ -399,7 +416,7 @@ class IBKR:
                 if len(spinoff) != 7:
                     logging.error(g_tr('StatementLoader', "Spin-off description miss some data ") + f"'{action}'")
                     continue
-                asset_id_old = self._parent.findAssetID(spinoff['symbol_old'], spinoff['isin_old'])
+                asset_id_old = JalDB().get_asset_id(spinoff['symbol_old'], isin=spinoff['isin_old'])
                 qty_old = int(spinoff['Y']) * action['quantity'] / int(spinoff['X'])
                 JalDB().add_corporate_action(action['accountId'], CorporateAction.SpinOff, action['dateTime'],
                                              action['transactionID'], asset_id_old,
@@ -415,7 +432,7 @@ class IBKR:
                     logging.error(g_tr('StatementLoader', "Spin-off description miss some data ") + f"'{action}'")
                     continue
                 description_b = action['description'][:parts.span(4)[0]] + isin_change['symbol_old'] + ".OLD, "
-                asset_b = self._parent.findAssetID(isin_change['symbol_old'], isin_change['isin_old'])
+                asset_b = JalDB().get_asset_id(isin_change['symbol_old'], isin=isin_change['isin_old'])
 
                 paired_record = list(filter(
                     lambda pair: pair['symbol'] == asset_b
@@ -460,7 +477,7 @@ class IBKR:
                     cnt += 1
                 else:  # Split together with ISIN change and there should be 2nd record available
                     description_b = action['description'][:parts.span(5)[0]] + split['symbol_old'] + ".OLD, "
-                    asset_b = self._parent.findAssetID(split['symbol_old'], split['isin_old'])
+                    asset_b = JalDB().get_asset_id(split['symbol_old'], isin=split['isin_old'])
 
                     paired_record = list(filter(
                         lambda pair: pair['symbol'] == asset_b
@@ -534,7 +551,7 @@ class IBKR:
 
         taxes = list(filter(lambda tr: tr['type'] == IBKRCashOp.TaxWithhold, cash))
         for tax in taxes:
-            cnt += self.loadIBWithholdingTax(tax)
+            cnt += self.applyWitholdingTax(tax)
 
         transfers = list(filter(lambda tr: tr['type'] == IBKRCashOp.DepositWithdrawal, cash))
         for transfer in transfers:
@@ -607,10 +624,6 @@ class IBKR:
                              interest['amount'], interest['description'], interest['tradeID'])
         return 1
 
-    def loadIBWithholdingTax(self, tax):
-        self.addWithholdingTax(tax['dateTime'], tax['accountId'], tax['symbol'], -tax['amount'], tax['description'])
-        return 1
-
     def loadIBFee(self, fee):
         JalDB().add_cash_transaction(fee['accountId'], self._parent.getAccountBank(fee['accountId']), fee['dateTime'],
                                      fee['amount'], PredefinedCategory.Fees, fee['description'])
@@ -644,43 +657,119 @@ class IBKR:
                                  pair_account, -cash['amount'], 0, 0, cash['description'])
         return 1
 
-    def addWithholdingTax(self, timestamp, account_id, asset_id, amount, note):
-        parts = re.match(IBKR.TaxNotePattern, note)
+    # Applies tax to matching dividend:
+    # if tax < 0: apply it to dividend without tax
+    # otherwise: it is a correction and there should be dividend with exactly the same tax that will be set to 0
+    def applyWitholdingTax(self, tax):
+        parts = re.match(IBKR.TaxFullPattern, tax['description'], re.IGNORECASE)
+        if not parts:
+            logging.warning(g_tr('StatementLoader', "*** MANUAL ENTRY REQUIRED ***"))
+            logging.warning(g_tr('StatementLoader', "Unhandled tax country pattern found: ") + f"{tax['description']}")
+            return 0
+        parts = parts.groupdict()
+        country_code = parts['country'].lower()
+        country_id = get_country_by_code(country_code)
+        update_asset_country(tax['symbol'], country_id)
+        description = parts['description']
+        previous_tax = tax['amount'] if tax['amount'] >= 0 else 0
+        new_tax = -tax['amount'] if tax['amount'] < 0 else 0
+
+        dividend_id = self.findDividend4Tax(tax['dateTime'], tax['accountId'], tax['symbol'],
+                                            previous_tax, new_tax, description)
+        if dividend_id is None:
+            logging.warning(g_tr('StatementLoader', "Dividend not found for withholding tax: ") +
+                            f"{tax}, {previous_tax}")
+            return 0
+        _ = executeSQL("UPDATE dividends SET tax=:tax WHERE id=:dividend_id",
+                       [(":dividend_id", dividend_id), (":tax", new_tax)], commit=True)
+        return 1
+
+    # Searches for divident that matches tax in the best way:
+    # - it should have exactly the same account_id and asset_id
+    # - tax amount withheld from dividend should be equal to provided 'tax' value
+    # - timestamp should be the same or within previous year for weak match of Q1 taxes
+    # - note should be exactly the same or contain the same key elements
+    def findDividend4Tax(self, timestamp, account_id, asset_id, prev_tax, new_tax, note):
+        DIV_ID = 0
+        TIMESTAMP = 1
+        AMOUNT = 2
+        NOTE = 3
+
+        # select all valid candidates
+        if datetime.utcfromtimestamp(timestamp).timetuple().tm_yday < 75:
+            # We may have wrong date in taxes before March, 15 due to tax correction
+            range_start = ManipulateDate.startOfPreviousYear(day=datetime.utcfromtimestamp(timestamp))
+            query = executeSQL("SELECT id, timestamp, amount, note FROM dividends "
+                               "WHERE type=:div AND timestamp>=:start_range AND account_id=:account_id "
+                               "AND asset_id=:asset_id AND ABS(tax-:tax)<0.0001 ORDER BY timestamp",
+                               [(":div", DividendSubtype.Dividend), (":start_range", range_start),
+                                (":account_id", account_id), (":asset_id", asset_id), (":tax", prev_tax)],
+                               forward_only=True)
+        else:
+            # For any other day - use exact time match
+            query = executeSQL("SELECT id, timestamp, amount, note FROM dividends "
+                               "WHERE type=:div AND timestamp=:timestamp AND account_id=:account_id "
+                               "AND asset_id=:asset_id AND ABS(tax-:tax)<0.0001 ORDER BY timestamp",
+                               [(":div", DividendSubtype.Dividend), (":timestamp", timestamp),
+                                (":account_id", account_id), (":asset_id", asset_id), (":tax", prev_tax)],
+                               forward_only=True)
+        indexes = range(query.record().count())
+        dividends = []
+        while query.next():
+            values = list(map(query.value, indexes))
+            dividends.append(values)
+        # Choose either Dividends or Payments in liue with regards to note of the matching tax
+        if IBKR.PaymentInLiueOfDividend in note.upper():
+            dividends = list(filter(lambda item: IBKR.PaymentInLiueOfDividend in item[NOTE], dividends))
+            # we don't check for full match as there are a lot of records without amount
+        else:
+            dividends = list(filter(lambda item: IBKR.PaymentInLiueOfDividend not in item[NOTE], dividends))
+            # Check for full match
+            for dividend in dividends:
+                if (dividend[TIMESTAMP] == timestamp) and (note.upper() == dividend[NOTE][:len(note)].upper()):
+                    return dividend[DIV_ID]
+        if len(dividends) == 0:
+            return None
+
+        # Chose most probable dividend - by amount, timestamp and description
+        parts = re.match(IBKR.TaxNotePattern, note, re.IGNORECASE)
         if not parts:
             logging.warning(g_tr('StatementLoader', "*** MANUAL ENTRY REQUIRED ***"))
             logging.warning(g_tr('StatementLoader', "Unhandled tax pattern found: ") + f"{note}")
-            return
-        dividend_note = parts.group(1) + '%'
-        country_code = parts.group(2).lower()
-        country_id = get_country_by_code(country_code)
-        update_asset_country(asset_id, country_id)
-        dividend_id = self.findDividend4Tax(timestamp, account_id, asset_id, dividend_note)
-        if dividend_id is None:
-            logging.warning(g_tr('StatementLoader', "Dividend not found for withholding tax: ") + f"{note}")
-            return
-        old_tax = readSQL("SELECT tax FROM dividends WHERE id=:id", [(":id", dividend_id)])
-        _ = executeSQL("UPDATE dividends SET tax=:tax WHERE id=:dividend_id",
-                       [(":dividend_id", dividend_id), (":tax", old_tax + amount)], commit=True)
-
-    def findDividend4Tax(self, timestamp, account_id, asset_id, note):
-        # Check strong match
-        id = readSQL("SELECT id FROM dividends WHERE type=:div AND timestamp=:timestamp "
-                     "AND account_id=:account_id AND asset_id=:asset_id AND note LIKE :dividend_description",
-                     [(":div", DividendSubtype.Dividend), (":timestamp", timestamp), (":account_id", account_id),
-                      (":asset_id", asset_id), (":dividend_description", note)])
-        if id is not None:
-            return id
-        # Check weak match
-        range_start = ManipulateDate.startOfPreviousYear(day=datetime.utcfromtimestamp(timestamp))
-        count = readSQL("SELECT COUNT(id) FROM dividends WHERE type=:div AND timestamp>=:start_range "
-                        "AND account_id=:account_id AND asset_id=:asset_id AND note LIKE :dividend_description",
-                        [(":div", DividendSubtype.Dividend), (":start_range", range_start), (":account_id", account_id),
-                         (":asset_id", asset_id), (":dividend_description", note)])
-        if count > 1:
-            logging.warning(g_tr('StatementLoader', "Multiple dividends match withholding tax"))
             return None
-        id = readSQL("SELECT id FROM dividends WHERE type=:div AND timestamp>=:start_range "
-                     "AND account_id=:account_id AND asset_id=:asset_id AND note LIKE :dividend_description",
-                     [(":div", DividendSubtype.Dividend), (":start_range", range_start), (":account_id", account_id),
-                      (":asset_id", asset_id), (":dividend_description", note)])
-        return id
+        parts = parts.groupdict()
+        note_prefix = parts['prefix']
+        note_suffix = parts['suffix']
+        try:
+            note_amount = float(parts['amount'])
+        except (ValueError, TypeError):
+            note_amount = 0
+        score = [DIV_ID] * len(dividends)
+        for i, dividend in enumerate(dividends):
+            parts = re.match(IBKR.DividendNotePattern, dividend[NOTE], re.IGNORECASE)
+            if not parts:
+                logging.warning(g_tr('StatementLoader', "*** MANUAL ENTRY REQUIRED ***"))
+                logging.warning(g_tr('StatementLoader', "Unhandled dividend pattern found: ") + f"{dividend[NOTE]}")
+                return None
+            parts = parts.groupdict()
+            try:
+                amount = float(parts['amount'])
+            except (ValueError, TypeError):
+                amount = 0
+            if abs(amount - note_amount) <= 0.000005:        # Description has very similar amount +++++
+                score[i] += 5
+            if dividend[TIMESTAMP] == timestamp:             # Timestamp exact match gives ++
+                score[i] += 2
+            if abs(0.1*dividend[AMOUNT] - new_tax) <= 0.01:  # New tax is 10% of dividend gives +
+                score[i] += 1
+            if parts['prefix'] == note_prefix:               # Prefix part of description match gives +
+                score[i] += 1
+            if parts['suffix'] == note_suffix:               # Suffix part of description match gives +
+                score[i] += 1
+        for i, vote in enumerate(score):
+            if (vote == max(score)) and (vote > 0):
+                return dividends[i][DIV_ID]
+        # Final check - if only one found, return it
+        if len(dividends) == 1:
+            return dividends[DIV_ID]
+        return None
