@@ -83,23 +83,30 @@ def executeSQL(sql_text, params=[], forward_only=True, commit=False):
 
 
 # -------------------------------------------------------------------------------------------------------------------
-# the same as executeSQL() but after query execution it takes first line of it
-# and packs all field values in a list to return
-def readSQL(sql_text, params=None, named=False):
+# the same as executeSQL() but after query execution it takes first line of query result and:
+# - returns None if no records were fetched by query
+# - otherwise returns first row of the query result:
+# named = False: result is packed into a list of field values
+# named = True: result is packet into a dictionary with field names as keys
+# - check_unique = True: checks that only 1 record was returned by query, otherwise returns None
+def readSQL(sql_text, params=None, named=False, check_unique=False):
     if params is None:
         params = []
     query = QSqlQuery(db_connection())
     query.setForwardOnly(True)
     if not query.prepare(sql_text):
-        logging.error(f"SQL prep: '{query.lastError().text()}' for query '{sql_text}' with params '{params}'")
+        logging.error(f"SQL prep: '{query.lastError().text()}' for query '{sql_text}' | '{params}'")
         return None
     for param in params:
         query.bindValue(param[0], param[1])
     if not query.exec_():
-        logging.error(f"SQL exec: '{query.lastError().text()}' for query '{sql_text}' with params '{params}'")
+        logging.error(f"SQL exec: '{query.lastError().text()}' for query '{sql_text}' | '{params}'")
         return None
     if query.next():
-        return readSQLrecord(query, named=named)
+        res = readSQLrecord(query, named=named)
+        if check_unique and query.next():
+            return None  # More then one record in result when only one expected
+        return res
     else:
         return None
 
@@ -134,6 +141,7 @@ def init_and_check_db(db_path):
     if not db.isValid():
         return LedgerInitError(LedgerInitError.DbDriverFailure)
     db.setDatabaseName(get_dbfilename(db_path))
+    db.setConnectOptions("QSQLITE_ENABLE_REGEXP=1")
     db.open()
     tables = db.tables(QSql.Tables)
     if not tables:
@@ -220,7 +228,7 @@ def get_account_name(account_id):
 
 # -------------------------------------------------------------------------------------------------------------------
 def get_country_by_code(country_code):
-    id = readSQL("SELECT id FROM countries WHERE code=:code", [(":code", country_code)])
+    id = readSQL("SELECT id FROM countries WHERE code=:code", [(":code", country_code)], check_unique=True)
 
     if id is None:
         query = executeSQL("INSERT INTO countries(name, code, tax_treaty) VALUES (:name, :code, 0)",
