@@ -2,9 +2,9 @@ import sys
 import os
 import logging
 import traceback
-from PySide2.QtCore import QTranslator
-from PySide2.QtWidgets import QApplication
-from jal.widgets.main_window import MainWindow, AbortWindow
+from PySide2.QtCore import Qt, QTranslator
+from PySide2.QtWidgets import QApplication, QMessageBox
+from jal.widgets.main_window import MainWindow
 from jal.db.helpers import init_and_check_db, LedgerInitError, get_language, update_db_schema
 
 
@@ -21,13 +21,13 @@ def main():
     os.environ['QT_MAC_WANTS_LAYER'] = '1'    # Workaround for https://bugreports.qt.io/browse/QTBUG-87014
 
     own_path = os.path.dirname(os.path.realpath(__file__)) + os.sep
-    db, error = init_and_check_db(own_path)
+    error = init_and_check_db(own_path)
 
     if error.code == LedgerInitError.EmptyDbInitialized:  # If DB was just created from SQL - initialize it again
-        db, error = init_and_check_db(own_path)
+        error = init_and_check_db(own_path)
 
     app = QApplication([])
-    language = get_language(db)
+    language = get_language()
     translator = QTranslator(app)
     language_file = own_path + "languages" + os.sep + language + '.qm'
     translator.load(language_file)
@@ -36,12 +36,17 @@ def main():
     if error.code == LedgerInitError.OutdatedDbSchema:
         error = update_db_schema(own_path)
         if error.code == LedgerInitError.DbInitSuccess:
-            db, error = init_and_check_db(own_path)
+            error = init_and_check_db(own_path)
 
-    if db is None:
-        window = AbortWindow(error)
+    if error.code != LedgerInitError.DbInitSuccess:
+        window = QMessageBox()
+        window.setAttribute(Qt.WA_DeleteOnClose)
+        window.setWindowTitle("JAL: Start-up aborted")
+        window.setIcon(QMessageBox.Critical)
+        window.setText(error.message)
+        window.setInformativeText(error.details)
     else:
-        window = MainWindow(db, own_path, language)
+        window = MainWindow(own_path, language)
     window.show()
 
     app.exec_()
