@@ -1,7 +1,7 @@
 import re
 import logging
 from datetime import date, datetime
-from jal.ui_custom.helpers import g_tr
+from jal.widgets.helpers import g_tr
 
 # standard header is "DLSG            DeclYYYY0102FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF", where YYYY is year of declaration
 HEADER_LENGTH = 60
@@ -183,7 +183,12 @@ class DLSG:
         'gb': '826',
         'de': '276',
         'cn': '156',
-        'fi': '246'
+        'fi': '246',
+        'nl': '528',
+        'gr': '300',
+        'bm': '060',
+        'br': '076',
+        'je': '832'
     }
 
     def __init__(self, only_dividends=False):
